#include "scene.h"
#include <iostream.h>
#include <fstream.h>
#include <float.h>
#include <string.h>
#include "FL/gl.h"

#include "m33.h"

Scene *scene;

Scene::Scene() {

  tmeshN = 0;
  int w = 640;
  int h = 480;
  fb = new FrameBuffer(20, 40, w, h);
  fb->label("SW rendering");
  phc = new PHC(fb->w, fb->h, 60.0f);
  ui = new GraphAppUI();
  ui->GraphAppW->position(800, 40);

  animationOn = false;
  animationStep = 0;

  // geometry
  tmeshN = 0;
  float maxDim;
  V3 newCenter;
  unsigned int color;

  maxDim = 75.0f;
  newCenter =  V3(50.0f, 0.0f, -200.0f);
  color = 0xFF00AAAA;
  tmesh[tmeshN] = new TMesh;
  tmesh[tmeshN]->Load("geo/chess.tris");
  tmesh[tmeshN]->ScaleToMaxDim(maxDim);
  tmesh[tmeshN]->TranslateTo(newCenter);
  tmesh[tmeshN]->SetColor(color);
  tmeshN++;
  
  maxDim = 75.0f;
  newCenter =  V3(0.0f, 0.0f, -100.0f);
  color = 0xFF0000AA;
  tmesh[tmeshN] = new TMesh;
  tmesh[tmeshN]->Load("geo/chess.tris");
  tmesh[tmeshN]->ScaleToMaxDim(maxDim);
  tmesh[tmeshN]->TranslateTo(newCenter);
  tmesh[tmeshN]->SetColor(color);
  tmeshN++;


  Show();
  Redraw();

}

void Scene::Show() {

  if (fb)
    fb->show();
  if (ui)
    ui->GraphAppW->show();

}

void Scene::UpdateScene() {

  V3 trans(1.0f, 0.0f, 0.0f);
  tmesh[1]->TranslateBy(trans);
  animationStep++;

}

void Scene::Redraw() {

  if (animationOn) {
    UpdateScene();
  }
  if (fb) {
    Render();
    fb->redraw();
  }
  if (animationOn) {
    Fl::check();
    Redraw();
  }

}


void Scene::DBG() {

  phc->IncreaseFocalLength(1.01f);
  Redraw();
  return;

  if (animationOn)
    animationOn = false;
  else
    animationOn = true;
  Redraw();
  return;

  M33 m;
  cerr << m;
  m[1][2] = 13.7f;
  cerr << m;
  return;

  V3 v(3.0f, 4.0f, 5.0f);
  V3 v2(1.0f, -1.0f, 3.0f);
  cerr << "v " << v << endl;
  cerr << "v2 " << v2 << endl;
  cerr << "v+v2 " << v+v2 << endl;
  cerr << "v-v2 " << v-v2 << endl;
  cerr << "v*v2 " << v*v2 << endl;
  cerr << "v length " << v.Length() << endl;
  cerr << "v normalized " << v.Normalized() << endl;

  return;

  if (fb) {
    fb->Clear(0xFFFFFFFF);
    float uf = 150.0f;
    float vf = 200.0f;
    int size = 53;
    fb->DrawPointSquare(uf, vf, size, 0xFF00FF00);
    fb->DrawPointDiagonalCross(uf, vf, size, 0xFF0000FF);
    float u1f = 170.4f;
    float v1f = 301.9f;
    fb->DrawSegment(uf, vf, u1f, v1f, 0xFFFF0000);
    unsigned int colors[2] = {0xFF00FF00, 0xFF0000FF};
    fb->MakeChecker(30, colors);
  }
  Redraw();

}

void Scene::Render() {

  if (!fb)
    return;

  unsigned int bgrColor = 0xFFFFFFFF;
  fb->Clear(bgrColor);

  for (int tmi = 0; tmi < tmeshN; tmi++) {
    tmesh[tmi]->Render(fb, phc);
  }

  return;


}

void Scene::MouseMove(int u, int v) {

  return;

  if (!fb)
    return;
  fb->DrawPointSquare(.5f+(float)u, .5f+(float)v, 5, 0xFF000000);
  Redraw();

}