#include "tmesh.h"
#include <fstream.h>
#include <strstrea.h>

TMesh::TMesh() {

  color = 0x00000000;
  vertsN = 0;
  verts = 0;
  trisN = 0;
  tris = 0;

}


void TMesh::Load(char *fileName) {

  ifstream ifs(fileName);
  if (ifs.ios::fail()) {
    cerr << "ERROR: cannot open: " << fileName << endl;
    return;
  }

  if (vertsN)
    delete verts;
  vertsN = 0;
  if (trisN)
    delete tris;
  trisN = 0;

  ifs >> vertsN;
  verts = new V3[vertsN];
  char lineBuffer[1000];
  ifs.getline(lineBuffer, 999);
  for (int vi = 0; vi < vertsN; vi++) {
    char lineBuffer[1000];
    ifs.getline(lineBuffer, 999);
    istrstream istr(lineBuffer);
    istr >> verts[vi];
  }
  cerr << "INFO: read " << vertsN << " vertices" << endl;


  ifs >> trisN;
  tris = new unsigned int[trisN*3];
  ifs.getline(lineBuffer, 999);
  for (int ti = 0; ti < trisN; ti++) {
    char lineBuffer[1000];
    ifs.getline(lineBuffer, 999);
    istrstream istr(lineBuffer);
    istr >> tris[3*ti+0] >> tris[3*ti+1] >> tris[3*ti+2];
  }

  cerr << "INFO: read " << trisN << " triangles" << endl;

  ifs.close();

}


void TMesh::Render(FrameBuffer *fb, PHC *phc) {

  for (int tri = 0; tri < trisN; tri++) {
    for (int vi = 0; vi < 3; vi++) {
      fb->DrawSegment3D(verts[tris[3*tri+vi]], verts[tris[3*tri+((vi+1)%3)]], 
        *phc, color);
    }
  }

}


V3 TMesh::FindCenter() {

  V3 ret(0.0f, 0.0f, 0.0f);
  for (int i = 0; i < vertsN; i++) {
    ret = ret + verts[i];
  }
  return ret / (float) vertsN;

}


void TMesh::ScaleToMaxDim(float maxDim) {

  int vi;
  BBox bbox(verts[0]);
  for (vi = 0; vi < vertsN; vi++) {
    bbox.AddPoint(verts[vi]);
  }

  float oldMaxDim = bbox.GetDiagonal();

  float scaleFactor = maxDim/oldMaxDim;

  V3 center = FindCenter();
  TranslateBy(center);
  ScaleBy(scaleFactor);
  TranslateBy(center*-1.0f);
  
}


void TMesh::TranslateBy(V3 translationVector) {

  for (int vi = 0; vi < vertsN; vi++) {
    verts[vi] = verts[vi] + translationVector;
  }

}

void TMesh::TranslateTo(V3 newCenter) {

  V3 center = FindCenter();
  TranslateBy(newCenter-center);

}

void TMesh::ScaleBy(float scf) {

  for (int vi = 0; vi < vertsN; vi++) {
    verts[vi] = verts[vi] * scf;
  }

}
