#include "v3.h"
#include <math.h>

V3::V3(float x, float y, float z) {

  xyz[0] = x;
  xyz[1] = y;
  xyz[2] = z;

}

V3::V3(float *_xyz) {

  xyz[0] = _xyz[0];
  xyz[1] = _xyz[1];
  xyz[2] = _xyz[2];

}

float& V3::operator[](int i) {

  return xyz[i];

}

ostream& operator<<(ostream& ostr, V3 &v) {

  return ostr << 
    v.xyz[0] << " " << 
    v.xyz[1] << " " << 
    v.xyz[2];

}

istream& operator>>(istream& istr, V3 &v) {

  return istr >> 
    v.xyz[0] >> 
    v.xyz[1] >> 
    v.xyz[2];

}

V3 V3::operator*(float scalar) {

  V3 ret;
  ret[0] = xyz[0]*scalar;
  ret[1] = xyz[1]*scalar;
  ret[2] = xyz[2]*scalar;
  return ret;

}

V3 V3::operator/(float scalar) {

  V3 ret;
  ret[0] = xyz[0]/scalar;
  ret[1] = xyz[1]/scalar;
  ret[2] = xyz[2]/scalar;
  return ret;

}


V3 V3::operator+(V3 op1) {

  V3 ret;
  ret[0] = xyz[0]+op1[0];
  ret[1] = xyz[1]+op1[1];
  ret[2] = xyz[2]+op1[2];
  return ret;

}

V3 V3::operator-(V3 op1) {

  V3 ret;
  ret[0] = xyz[0]-op1[0];
  ret[1] = xyz[1]-op1[1];
  ret[2] = xyz[2]-op1[2];
  return ret;

}

float V3::operator*(V3 op1) {

  float ret = 
    xyz[0]*op1[0]+
    xyz[1]*op1[1]+
    xyz[2]*op1[2];
  return ret;

}

float V3::Length() {

  V3 v = (*this);
  return sqrtf(v*v);

}

V3 V3::Normalized() {
  
  V3 ret = (*this)/Length();
  return ret;

}

V3 V3::operator^(V3 op2) {

  V3 ret;
  ret[0] = xyz[1]*op2[2]-xyz[2]*op2[1];
  ret[1] = xyz[2]*op2[0]-xyz[0]*op2[2];
  ret[2] = xyz[0]*op2[1]-xyz[1]*op2[0];
  return ret;

}


BBox::BBox(V3 firstPoint) {

  corners[0] = firstPoint;
  corners[1] = firstPoint;

}


void BBox::AddPoint(V3 newPoint) {

  for (int i = 0; i < 3; i++) {
    if (newPoint[i] < corners[0][i])
      corners[0][i] = newPoint[i];
    if (newPoint[i] > corners[1][i])
      corners[1][i] = newPoint[i];
  }

}

float BBox::GetDiagonal() {

  return (corners[1]-corners[0]).Length();

}